function [time_cell, params_cell, code] = ...
    retrieve_t(diag_name, shot_number, sub_shot, channel_number, varargin)
% Make time arrays for diagnostics's channels, or Retrieve DTS's parameter on SHOT.
%   [time_cell, params_cell, code] = retrieve_t( diag_name, ...
%               shot_number, sub_shot, channel_number, ...)
%
%       diag_name(text)     : name of diagnostics or DTS
%           DTS name's format are 'DTS:hostID','DTS:hostID:',
%                               'DTS:hostID:moduleID'
%           If it is diagnostics name, this function make time array,
%           otherwise this function retrieve DTS's parameter.
%       shot_number(numeric): SHOT number
%       sub_shot(numeric)   : SHOT sub number
%       channel_number(numeric, [vector],text)
%                           : channel number
%           range format is "n:m"(vector), single format is "n"(numeric), 
%            and both(e.g. [1,3,5:7]).
%           "0" is all channels.
%           (text)          : signal tag name (e.g. 'sig-tag1')
%
%
%       time_cell(cell)     : time data
%       params_cell(cell)   : parameter data
%       code(numeric)       : result code
%           If code is 0, the result is success; otherwise is an error.
%
% OPTIONS :
%         'double'          : data is double precision.( default )
%         'single'          : data is single precision.
%         'float'           : same as 'single'
%         'silent'          : do not display messages.
%         'wait'            : wait for the appearance of data
%         'timeout',timeout_sec(numeric)
%                           : seconds of timeout for wait
%                             The default is 180 sec.
%
% Examples :
%   (a) Make time array for diagnostics's channel with DTS information on SHOT.
%
%   >> [time_cell,~,code] = retrieve_t('RADH',98000,1,2);
%   ---DTS Infomation---
%   Index Server Name  = dasindex.lhd.nifs.ac.jp
%   Shot Number        =      98000
%   Sub Shot Number    =          1
%   DTS source         = VME
%   DTS host ID        = 133.75.175.172
%   DTS module ID      = DMOD0
%   DTS trigger ch     =          5
%   DTS clock ch       =          2
%   DTS user define    =          1
%   Time array size    =    1000000
%   Cycle = [0.00001000] sec
%   Delay = [0.00000000] sec
%   >>
%
%   (b) Retrieve DTS's parameter on SHOT.
%
%   >>[~,param,code] = retrieve_t('DTS:VME01',98000,1,0);
%   ---DTS Infomation---
%   Index Server Name  = dasindex.lhd.nifs.ac.jp
%   Shot Number        =      98000
%   Sub Shot Number    =          1
%   DTS source         = VME01:DMOD0
%   Channel Range      = [   1  -    8 ]
%   >>printparam(param);
%                           Name                    Value     Type
%     1                     Shot                    98000    int32
%     1                  SubShot                        1    int32
%   ...........
%
% Last update : 2017-12-01
%
% See also
%     retrieve_td printParam getParam retrieve


  ip = inputParser;
  ip.KeepUnmatched = true;

  ip.addRequired('diag_name', @ischar);
  ip.addRequired('shot_number', @(x)isnumeric(x) && isscalar(x) && x > 0);
  ip.addRequired('sub_shot',@(x)isnumeric(x) && isscalar(x) && x >= 0);
%ip.addRequired('channel_number',@(x)isnumeric(x) && all(x>=0));

%ip.parse( diag_name, shot_number, sub_shot, channel_number );
  ip.parse( diag_name, shot_number, sub_shot );

  wait_time_sec = 0;
  flg_timeout = false;
  time_dbl = true;
  verbose = true;
  nvarargin = length(varargin);
  arg_check = true;
  err_arg = '';

  for idx = 1:nvarargin

    if isempty(varargin{idx})
      continue;
    elseif flg_timeout
      if isnumeric(varargin{idx}) && isscalar(varargin{idx})
        wait_time_sec = varargin{idx};
        flg_timeout = false;
        continue
      else
        arg_check = false;
        break
      end
    end
    
    if ~ischar(varargin{idx}) 
      err_arg = strcat(err_arg, [' arg#' num2str(idx+4)]);
      arg_check = false;
      break
    end
    switch varargin{idx}
      case 'double'
        time_dbl = true;
      case {'single','float'}
        time_dbl = false;
      case 'silent'
        verbose = false;
      case 'timeout'
        flg_timeout = true;
      case 'wait'
        if wait_time_sec == 0
          wait_time_sec = 180;
        end
      otherwise
        arg_check = false;
        err_arg = strcat(err_arg, [' ''' varargin{idx} '''']);
    end
  end
  if nargout > 0
    time_cell = cell(0);
  end
  if nargout > 1
    params_cell = cell(0);
  end
  if not(ischar(channel_number) || ( isnumeric(channel_number) && all(channel_number>=0) ) )
    if verbose
      disp('<Error: illegal channles>');
      disp('Type "help retrieve_t".');
    end
    code = -2;
    return;
  end

  if( ~arg_check ) 
    if verbose
      disp(['<Error: illegal argument ' err_arg ' >']);
      disp('Type "help retrieve_t".');
    end
    code = -2;
    return;
  end
  sp = strfind(diag_name,'DTS:');
  if 1 == length(sp) && 1 == sp(1) 
    sp = strfind(diag_name,':');
    dts_name = diag_name(sp(1)+1:length(diag_name));
  else
    dts_name = '';
  end
  if isempty(dts_name)
% Make time array for diagnostics's channel with DTS information on SHOT.
    [time_cell, params_cell, code] ...
      = retrieve_time(diag_name, shot_number, sub_shot, channel_number, ...
          time_dbl, wait_time_sec, verbose );
  else
% Retrieve DTS's parameter on SHOT.
    if ischar(channel_number) 
      if verbose
        disp('<Error: illegal channles>');
        disp('Type "help retrieve_t".');
      end
      code = -2;
      return;
    end
    [params_cell, code] ...
      = retrieve_dts(dts_name, shot_number, sub_shot, channel_number,  verbose );
  end
end

function [time_cell, params_cell, code] = ...
  retrieve_time(diag_name, shot_number, sub_shot, channel_number, time_dbl, wait_time_sec, verbose )
% Make time array for diagnostics's channel with DTS information on SHOT.
%   [time_array_ptr, params_cell, code] = ...
%       retrieve_time( diag_name, shot_number, channel_number, ...
%                      wait_time_sec, verbose )
%
%       diag_name(text)     : diagnostics name
%       shot_number(numeric): SHOT number
%       sub_shot(numeric)   : SHOT sub number
%       channel_number(numeric)
%                           : channel number(single only)
%       time_dbl(logical)   : true is double precision array.
%                             false is single precision array. 
%       wait_time_sec(numeric)
%                           :
%       verbose(logical)    : true is output messages to a display.
%                             false is donot output messages to a display. 
%
%       time_cell(cell)     : time data
%       params_cell(cell)   : parameter data
%       code(numeric)       : result code
%             if code is 0, the result is success; otherwise is an error.
%

  if nargout > 0
    time_cell = cell(0);
  end
  if nargout > 1
    params_cell = cell(0);
  end

  [rtd, index_server] ...
       = mRetrieveOpen( diag_name, shot_number, sub_shot, '', '', ...
         wait_time_sec, 0,"",verbose);
  if rtd < 0
    code=rtd;
    return;
  end
  [code last_channel_number syy smm sdd shour smin ssec trans_server] ...
      = mRetrieveShotInfo( rtd, verbose);
  if code < 0
    return
  end

  if ischar(channel_number)
    type_ch = 0;
  else
    type_ch = 2;
    if any(channel_number > last_channel_number )
      code = -1;
      if verbose
        disp('Illegal Channel number.\n');
      end
      return;
    end
    if 0 == channel_number(1)
      channel_number = 1:last_channel_number;
    end
  end
  
  if time_dbl
    is_dbl = 1;
  else
    is_dbl = 0;
  end

  if verbose
    fprintf('---DTS Infomation---\n');
    fprintf('Index Server Name  = %s\n', index_server);
    fprintf('Shot Number        = %10d\n', shot_number);
    fprintf('Sub Shot Number    = %10d\n', sub_shot);
  end
  if shot_number < 56221
    need_dtslink = 0;
  else
    need_dtslink = 1;
  end

  idx = 1;
  for chno = channel_number

    pre_samples = 0;
    sampling_interval = 0;
    if need_dtslink == 0 
      [code o_dts_source o_dts_host o_dts_mod o_dts_trg_no o_dts_clk_no ...
        dts_user_define time_array_size clock_source pre_samples ...
        sampling_interval o_clk_dts_id o_clk_dts_moduleid] ...
        = cRetrieveGetDTSinfox( diag_name, shot_number, sub_shot, ...
                                chno, chno, index_server, verbose);
     if code ~= 0
       break;
     end
     
    end
    
    if type_ch == 0
      [code real_channel ret_shot ret_sub ...
        dts_source, dts_host, dts_mod, dts_trg_ch, ...
        clk_source, clk_host, clk_mod, dts_clk_ch, ...
        dts_user_define, time_array_size, ext_or_int, ...
        internal_clock, s_sampling_interval, s_pre_samples, last_count] ...
          = cRetrieveGetDTSinfoFromRetrieve_ex( rtd, channel_number, need_dtslink, verbose );
    else
          [code real_channel ret_shot ret_sub ...
        dts_source, dts_host, dts_mod, dts_trg_ch, ...
        clk_source, clk_host, clk_mod, dts_clk_ch, ...
        dts_user_define, time_array_size, ext_or_int, ...
        internal_clock, s_sampling_interval, s_pre_samples, last_count] ...
          = cRetrieveGetDTSinfoFromRetrieve_ex( rtd, chno, need_dtslink, verbose );
    end
    if code ~= 0
      break;
    end
    if need_dtslink == 0 
     dts_source = o_dts_source;
     dts_host   = o_dts_host;
     dts_mod    = o_dts_mod;
     clk_source = o_dts_source;
     clk_host   = o_dts_host;
     if( 0 != o_clk_dts_moduleid.length() )   
      clk_mod = o_clk_dts_moduleid;
     else
      clk_mod = o_dts_mod;
     end
     dts_trg_ch = num2str(o_dts_trg_no);
     dts_clk_ch = num2str(o_dts_clk_no);
    end    
    if 0 < pre_samples
      s_pre_samples = num2str(pre_samples, 10);
    end
    if 0 < sampling_interval
      s_sampling_interval = num2str(sampling_interval, 10);
    end
    if verbose
      fprintf('\n');
      fprintf('DTS source         = %s\n', dts_source);
      fprintf('DTS host ID        = %s\n', dts_host);
      fprintf('DTS module ID      = %s\n', dts_mod);
      fprintf('DTS trigger ch     = %s\n', dts_trg_ch);
      fprintf('CLK source         = %s\n', clk_source);
      fprintf('CLK host ID        = %s\n', clk_host);
      fprintf('CLK module ID      = %s\n', clk_mod);
      fprintf('DTS clock ch       = %s\n', dts_clk_ch);
      fprintf('Clock Source       = %s\n', ext_or_int);
      fprintf('DTS user define    = %10d\n', dts_user_define);
      fprintf('Time array size    = %10d\n', time_array_size);
    end

    [code clock_cycle trigger_timing rtn_user_define time_array  ] ...
      = cRetrieveGetDTSdatax2_ex( index_server, ...
         dts_source, dts_host, dts_mod, dts_trg_ch, ...
         clk_source, clk_host, clk_mod, dts_clk_ch, ...
         dts_user_define, time_array_size, ret_shot, ret_sub, ...
         ext_or_int, internal_clock, s_sampling_interval, s_pre_samples, ...
         last_count, 1, is_dbl, verbose);
    if code ~= 0 
      break;
    end
    
    if verbose 
      fprintf('Cycle   = [%f] sec\n',clock_cycle);
      fprintf('Delay   = [%f] sec\n',trigger_timing);
    end
    
    time_cell{idx,1} = time_array;
    if nargout > 1
      params(13,1) = struct('Pname',[],'Pvalue',[],'Ptype',[]);

      params(1,1) = struct('Pname', 'DTSsource',    'Pvalue', dts_source, 'Ptype', 1);
      params(2,1) = struct('Pname', 'DTShostID',    'Pvalue', dts_host, 'Ptype', 1);
      params(3,1) = struct('Pname', 'DTSmoduleID',  'Pvalue', dts_mod, 'Ptype', 1);
      params(4,1) = struct('Pname', 'TriggerCh',    'Pvalue', dts_trg_ch, 'Ptype', 4);
      params(5,1) = struct('Pname', 'CLKsource',    'Pvalue', clk_source, 'Ptype', 1);
      params(6,1) = struct('Pname', 'CLKhostID',    'Pvalue', clk_host, 'Ptype', 1);
      params(7,1) = struct('Pname', 'CLKmoduleID',  'Pvalue', clk_mod, 'Ptype', 1);
      params(8,1) = struct('Pname', 'ClockCh',      'Pvalue', dts_clk_ch, 'Ptype', 4);
      params(9,1) = struct('Pname', 'ClockSource',  'Pvalue', ext_or_int, 'Ptype', 1);
      params(10,1) = struct('Pname', 'TimeArraySize','Pvalue', num2str(time_array_size), 'Ptype', 7);

      if time_dbl
        params(11,1) = struct('Pname', 'ArrayDataType',   'Pvalue', 'Double', 'Ptype', 1);
        params(12,1) = struct('Pname', 'ClockCycle(sec)', 'Pvalue', num2str(clock_cycle, 15), 'Ptype', 6);
        params(13,1) = struct('Pname', 'StartTiming(sec)','Pvalue', num2str(trigger_timing, 15), 'Ptype', 6);
      else
        params(11,1) = struct('Pname', 'ArrayDataType',   'Pvalue', 'Single', 'Ptype', 1);
        params(12,1) = struct('Pname', 'ClockCycle(sec)', 'Pvalue', num2str(clock_cycle, 7), 'Ptype', 5);
        params(13,1) = struct('Pname', 'StartTiming(sec)','Pvalue', num2str(trigger_timing, 7), 'Ptype', 5);
      end
      params_cell{idx,1} = params;
    end
    if type_ch == 0
      break;
    end
    idx = idx + 1;
  end
  mRetrieveClose(rtd);
  return
end

function [params_cell, code] = retrieve_dts(dts_name, shot_number, sub_shot, channel_number,  verbose )
% Retrieve DTS's parameter on SHOT.
%   [params_cell, code] = retrieve_dts( dts_name, ...
%                         shot_number, sub_shot, channel_number,  verbose )
%
%       dts_name(text)      : DTS name ('hostID','hostID:','hostID:moduleID' )
%       shot_number(numeric): SHOT number
%       sub_shot(numeric)   : SHOT sub number
%       channel_number(numeric)
%                           : channel number
%           range format is "n:m", single format is "n", 0 is all channels.
%       verbose(logical)    : true is output messages to a display.
%                             false is donot output messages to a display. 
%
%
%       params_cell(cell)   : DTS's parameter, 1-cell is 1-channel.
%       code(numeric)       : result code
%             if code is 0, the result is success; otherwise is an error.
%
  params_cell = cell(0);
  mod_name = '';
  sp = strfind(dts_name,':');
  if isempty(sp)
    host_name = dts_name;
  else
    host_name = dts_name(1:sp(1)-1);
    if 1 == length(sp)
      mod_name = dts_name(sp(1)+1:length(dts_name));
    else
      mod_name = dts_name(sp(1)+1:sp(2)-1);
    end
  end
  if isempty(mod_name)
    mod_name = 'NONAME';
  end
  start_ch = min(channel_number);
  end_ch = max(channel_number);
  if 0 == start_ch
    start_ch = 1;
    end_ch = 0;
  end
  index_server=getenv("INDEXSERVERNAME");

  [code param_count ret_start_ch ret_end_ch x_mod_name] = ...
          cRetrieveGetDTSParametersCount( index_server, host_name, mod_name, ...
           shot_number, sub_shot, start_ch, end_ch, verbose);
  if code ~= 0 
   return;
  end

  if 0 == end_ch 
    channel_number = 1:ret_end_ch;
  end
  if verbose
    fprintf('---DTS Infomation---\n');
    fprintf('Index Server Name  = %s\n', index_server);
    fprintf('Shot Number        = %10d\n', shot_number);
    fprintf('Sub Shot Number    = %10d\n', sub_shot);
    fprintf('DTS source         = %s:%s\n', host_name, x_mod_name);
    fprintf('Channel Range      = [ %3d  -  %3d ]\n', min(channel_number), max(channel_number));
  end

  params_cell = cell(length(channel_number),1);
  idx = 1;
  for chno = channel_number
    [code params] = cRetrieveGetDTSParameters2( index_server, ...
         host_name, mod_name, shot_number, sub_shot, chno, param_count, verbose);
    if code ~= 0 
      return;
    end
              
    params_cell{idx,1}=params;
    idx = idx+1;
  end

end
