#include <octave/oct.h>
#include "dbstore-oct.h"

/**
 DBSTORE_API int dbsOpenFrame2(int dbs_d, int ch_no,
		unsigned int x_size,unsigned int y_size, char* data_type, int sw_accept_size );
 */
DEFUN_DLD (dbsOpenFrame, args, nargout,
" Open Process for write frame data in channel. \n"
"   code = dbsOpenFrame( dbs_des, channel_number, frame_x, frame_y, ... \n"
"          image_type, ...) \n"
" \n"
"       dbs_des(numeric)    : opened dbStore's descriptor id \n"
"       channel_number(numeric) \n"
"                           : channel number \n"
"       frame_x(numeric)    : x size of frame \n"
"       frame_y(numeric)    : y size of frame \n"
"       image_type(text)    : type of frame data. \n"
"           Please refer to 'Note :' for the details. \n"
" \n"
"       code(numeric)       : status, 0 is success, other is error. \n"
" \n"
"  OPTIONS : \n"
"       'accept_sizes'      : The size of frame data is accepted also except X*Y size.  \n"
"       'verbose'           : If an error occurs, output messages to display. \n"
" \n"
"  Example : \n"
"       des = dbsOpen( .... ); \n"
"       if des > 0 \n"
"          while true \n"
"              ch=1; \n"
"              code = dbsOpenFrame(des, ch, 640,480,'RGBA'); \n"
"              if code ~= 0  \n"
"                 break; \n"
"              end \n"
"              code = dbsWriteFrame(des, ch, 1,frame_array);  \n"
"              if code ~= 0  \n"
"                 break; \n"
"              end \n"
"  \n"
"              .... \n"
"              code = dbsCloseFrame(des, ch, param);  \n"
"              if code ~= 0  \n"
"                 break; \n"
"              end \n"
"              .... \n"
"          end \n"
"          .... \n"
"          if 0 == code  \n"
"               dbsClsoe(des); \n"
"          else \n"
"               dbsAbort(des); \n"
"          end \n"
"       end \n"
" \n"
"  Note : \n"
"      image_type(text) has the next kind. \n"
"          'INT8'  : 8bit integer. \n"
"          'INT16' : 16bit integer. \n"
"          'INT32' : 32bit integer. \n"
"          'INT64' : 64bit integer. \n"
"          'FLT32' : single precision. \n"
"          'FLT64' : double precision. \n"
"          'GRAYn' : n bits gray image.(n:8-16) ex.) GRAY8 \n"
"          'GREYn' : same 'GRAYn' \n"
"          'RGB'   : 24 bit color image. \n"
"                    There is a meaning in letter order of 'R','G','B' \n"
"          'RGBA'  : 32 bit color image. \n"
"                    There is a meaning in letter order of 'R','G','B','A' \n"
" \n"
"  Last update : 2018-05-30 \n"
" \n"
"  See also \n"
"      dbsOpen(.oct) dbsWrite(.oct) dbsClose(.oct) dbsAbort(.oct) \n"
"      initParamList(.m) setParam(.m) printParam(.m) \n"
"      dbsOpenFrame(.oct) dbsWriteFrame(.oct) dbsCloseFrame(.oct) \n" 
"      dbsWriteFrames(.oct) dbsMessage(.oct) dbstoreVersion(.oct) dbstore.txt \n"
           )
{
  const int MIN_IN_ARG = 5;
  const int NUM_OUT_ARG = 1;
  const std::string funcn = "dbsOpenFrame";

  int dbsd;
  int ch_no;
  int frame_x;
  int frame_y;
  std::string image_type = "";
  
  std::string opt_key = "";
  bool opt_verbose = false;
  int acceptsizes = 0;

  int nargin = args.length();
  int code = 0;
  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG ){ code = OCT_IF_ERR_ARG_NUM_INPUT_ARG; break; }
    // check 0. nargout
    if( nargout > NUM_OUT_ARG ){ code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG; break;}
    
    // check 1.dbsd
    if( 0 != (code=get_arg_int( args(0), dbsd )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" dbs_des:" << dbsd << "\n";
    // check 2.channel_number
    if( 0 != (code=get_arg_int( args(1), ch_no )) )  {
      code += OCT_IF_ERR_ARG2;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" channel_number:" << ch_no << "\n";
    // check 3.frame_x
    if( 0 != (code=get_arg_int( args(2), frame_x )) )  {
      code += OCT_IF_ERR_ARG3;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" frame_x:" << frame_x << "\n";
    // check 4.frame_y
    if( 0 != (code=get_arg_int( args(3), frame_y )) )  {
      code += OCT_IF_ERR_ARG4;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" frame_y:" << frame_y << "\n";
    // check 5.image_type
    if( 0 != (code=get_arg_string( args(4), image_type )) ) {
      code += OCT_IF_ERR_ARG5; break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" image_type:" << image_type << "\n";
      
    break;
  } // End of for( ;; )
  // Option
  int xcode= 0;
  for( int x = MIN_IN_ARG; x < nargin; x++ ) {
    if( 0 == (xcode=get_arg_string( args(x), opt_key )) ) {
      if( opt_key == "verbose" ) {
        opt_verbose = true;
      }
      else if( opt_key == "accept_sizes" ) {
        acceptsizes = 1;
      }
      else {
        if( 0 == code )
          code = OCT_IF_ERR_ARG_OPT(x+1) + OCT_IF_ERR_ARG_NO_SUPPORTED_OPTION;
      }
    }
    else {
      if( 0 == code )
        code = OCT_IF_ERR_ARG_OPT(x+1) + xcode;
    }
  }
  
  if( 0 == code ) {
    code = dbsOpenFrame2( dbsd, ch_no, frame_x,frame_y, (char*)image_type.c_str(), acceptsizes);
  }
  octave_value_list ret_vals;
  ret_vals(0) = code;
  if( 0 != code ) {
    if( opt_verbose || retrieve_debug ) octave_dbs_error_message(code, funcn);
  }
  return ret_vals;
}
