#include <octave/oct.h>
#include <octave/dMatrix.h>
#include <octave/uint32NDArray.h>
#include "dbstore-oct.h"

/**
DBSTORE_API int dbsWriteFrame(int dbs_d, int ch_no, int frame_no,
			unsigned int data_length, void* data)
 */
DEFUN_DLD (dbsWriteFrame, args, nargout,
" Write a frame data in channel. \n"
"   code = dbsWriteFrame( dbs_des, channel_number, frame_number, ...\n"
"          frame_array, ...)\n"
"\n"
"       dbs_des(numeric)    : opened dbStore's descriptor id \n"
"       channel_number(numeric) \n"
"                           : channel number \n"
"       frame_number(numeric) \n"
"                           : frame number \n"
"       frame_array(numeric) \n"
"                           : data of a frame \n"
" \n"
"       code(numeric)       : status, 0 is success, other is error. \n"
" \n"
"  OPTIONS : \n"
"       'verbose'           : If an error occurs, output messages to display. \n"
" \n"
"  Example : \n"
"       des = dbsOpen( .... ); \n"
"       if des > 0 \n"
"          while true \n"
"              ch=1; \n"
"              code = dbsOpenFrame(des, ch, 640,480,'RGBA'); \n"
"              if code ~= 0  \n"
"                 break; \n"
"              end \n"
"              frame_array=zeros(640,480,'uint32'); \n"
"              code = dbsWriteFrame(des, ch, 1,frame_array);  \n"
"              if code ~= 0  \n"
"                 break; \n"
"              end \n"
"  \n"
"              .... \n"
"              code = dbsCloseFrame(des, ch, param);  \n"
"              if code ~= 0  \n"
"                 break; \n"
"              end \n"
"              .... \n"
"          end \n"
"          .... \n"
"          dbsClsoe(des); \n"
"       end \n"
" \n"
"  Last update : 2017-12-01 \n"
" \n"
"  See also \n"
"      dbsOpen dbsWrite dbsClose dbsAbort dbsMessage dbstoreVersion \n"
"      dbsOpenFrame dbsWriteFrames dbsCloseFrame dbstore.txt \n"
           )
{
  const int MIN_IN_ARG = 4;
  const int NUM_OUT_ARG = 1;
  const std::string funcn = "dbsWriteFrame";

  int dbsd;
  int ch_no;
  int frame_no;
  octave_value frame_data_val;
  
  std::string opt_key;
  bool opt_verbose = false;
  
  int nargin = args.length();
  int code = 0;
  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG ){ code = OCT_IF_ERR_ARG_NUM_INPUT_ARG; break; }
    // check 0. nargout
    if( nargout > NUM_OUT_ARG ){ code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG; break;}
    
    // check 1.dbsd
    if( 0 != (code = get_arg_int( args(0), dbsd )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" dbs_des:" << dbsd << "\n";
    // check 2.channel_number
    if( 0 != (code = get_arg_int( args(1), ch_no )) )  {
      code += OCT_IF_ERR_ARG2;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" channel_number:" << ch_no << "\n";
    // check 3.frame_number
    if( 0 != (code = get_arg_int( args(2), frame_no )) )  {
      code += OCT_IF_ERR_ARG3;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" frame_number:" << frame_no << "\n";
    // check 4.farme_data(matrix)
    if(  !args(3).is_matrix_type() ) {
      code = OCT_IF_ERR_ARG4 + OCT_IF_ERR_TYPE_MISSMATCH_ARRAY;   break;
    }
    frame_data_val = args(3);
    break;
  } // End of for( ;; )
  // Option
  int xcode = 0;
  for( int x = MIN_IN_ARG; x < nargin; x++ ) {
    if( 0 == (xcode=get_arg_string( args(x), opt_key )) ) {
      if( opt_key == "verbose" ) {
        opt_verbose = true;
      }
      else {
        if( 0 == code )
          code = OCT_IF_ERR_ARG_OPT(x+1) + OCT_IF_ERR_ARG_NO_SUPPORTED_OPTION;
      }
    }
    else {
        if( 0 == code ) code = OCT_IF_ERR_ARG_OPT(x+1) + xcode;
    }
  } // End of Option
  
  unsigned char* byte_data = NULL;
  size_t byte_size = 0;
  for( ; 0 == code;  ) {
    if( frame_data_val.is_uint8_type() ) {
      uint8NDArray frame_data = frame_data_val.uint8_array_value();
      byte_size = frame_data.byte_size();
      if( retrieve_debug ) octave_stdout << byte_size <<":is_uint8_type\n";
      byte_data = new unsigned char[byte_size];
      if( NULL ==  byte_data ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break; }
   
      size_t cnt = frame_data.numel();
      unsigned char* data = (unsigned char*) byte_data;
      for(size_t n=0;n<cnt; n++) {        data[n] = frame_data(n); }
    }
    else if( frame_data_val.is_uint16_type() ) {
      uint16NDArray frame_data = frame_data_val.uint16_array_value();
      byte_size = frame_data.byte_size();
      if( retrieve_debug ) octave_stdout << byte_size <<":is_uint16_type\n";
      byte_data = new unsigned char[byte_size];
      if( NULL ==  byte_data ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break; }
   
      size_t cnt = frame_data.numel();
      unsigned short* data = (unsigned short*) byte_data;
      for(size_t n=0;n<cnt; n++) {        data[n] = frame_data(n); }
    }
    else if( frame_data_val.is_uint32_type() ) {
      uint32NDArray frame_data = frame_data_val.uint32_array_value();
      byte_size = frame_data.byte_size();
      byte_data = new unsigned char[byte_size];
      if( NULL ==  byte_data ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break; }
   
      size_t cnt = frame_data.numel();
      unsigned int* data = (unsigned int*) byte_data;
      for(size_t n=0;n<cnt; n++) {        data[n] = frame_data(n); }
    }
    else if( frame_data_val.is_uint64_type() ) {
      uint64NDArray frame_data = frame_data_val.uint64_array_value();
      byte_size = frame_data.byte_size();
      byte_data = new unsigned char[byte_size];
      if( NULL ==  byte_data ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break; }
   
      size_t cnt = frame_data.numel();
      uint64_t* data = (uint64_t*) byte_data;
      for(size_t n=0;n<cnt; n++) {        data[n] = frame_data(n); }
    }
    else if( frame_data_val.is_int8_type() ) {
      int8NDArray frame_data = frame_data_val.int8_array_value();
      byte_size = frame_data.byte_size();
      byte_data = new unsigned char[byte_size];
      if( NULL ==  byte_data ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break; }
   
      size_t cnt = frame_data.numel();
      signed char* data = (signed char*) byte_data;
      for(size_t n=0;n<cnt; n++) {        data[n] = frame_data(n); }
    }
    else if( frame_data_val.is_int16_type() ) {
      int16NDArray frame_data = frame_data_val.int16_array_value();
      byte_size = frame_data.byte_size();
      byte_data = new unsigned char[byte_size];
      if( NULL ==  byte_data ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break; }
   
      size_t cnt = frame_data.numel();
      short* data = (short*) byte_data;
      for(size_t n=0;n<cnt; n++) {        data[n] = frame_data(n); }
    }
    else if( frame_data_val.is_int32_type() ) {
      int32NDArray frame_data = frame_data_val.int32_array_value();
      byte_size = frame_data.byte_size();
      byte_data = new unsigned char[byte_size];
      if( NULL ==  byte_data ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break; }
   
      size_t cnt = frame_data.numel();
      int* data = (int*) byte_data;
      for(size_t n=0;n<cnt; n++) {        data[n] = frame_data(n); }
    }
    else if( frame_data_val.is_int64_type() ) {
      int64NDArray frame_data = frame_data_val.int64_array_value();
      byte_size = frame_data.byte_size();
      byte_data = new unsigned char[byte_size];
      if( NULL ==  byte_data ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break; }
   
      size_t cnt = frame_data.numel();
      int64_t* data = (int64_t*) byte_data;
      for(size_t n=0;n<cnt; n++) {        data[n] = frame_data(n); }
    }
    else if( frame_data_val.is_double_type() ) {
      FloatNDArray frame_data = frame_data_val.float_array_value();
      byte_size = frame_data.byte_size();
      byte_data = new unsigned char[byte_size];
      if( NULL ==  byte_data ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break; }
   
      size_t cnt = frame_data.numel();
      float* data = (float*) byte_data;
      for(size_t n=0;n<cnt; n++) {        data[n] = frame_data(n); }
    }
    else if( frame_data_val.is_single_type() ) {
      NDArray frame_data = frame_data_val.array_value();
      byte_size = frame_data.byte_size();
      byte_data = new unsigned char[byte_size];
      if( NULL ==  byte_data ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break; }
   
      size_t cnt = frame_data.numel();
      double* data = (double*) byte_data;
      for(size_t n=0;n<cnt; n++) {        data[n] = frame_data(n); }
    }
    else {
      code = OCT_IF_ERR_ARG_DATA_ARRAY_NOT_NUMREIC;
      break;
    }
    break;
  } // End for( ;; )

  if( 0 == code ) {
    code = dbsWriteFrame( dbsd, ch_no, frame_no, (unsigned int)byte_size, byte_data);
  }
  if( NULL != byte_data ) delete [] byte_data;
      
  octave_value_list ret_vals;
  ret_vals(0) = code;
  if( 0 != code ) {
    if( opt_verbose || retrieve_debug)  octave_dbs_error_message(code, funcn);
  }
  return ret_vals;
}
