#include <octave/oct.h>
#include <octave/oct-map.h>
#include <octave/uint32NDArray.h>
#include "oct_if.h"
#include "dbstore.h"

#ifdef __linux__
#include "limits.h"
#endif

int get_arg_int64( const octave_value& arg, int64_t& val)
{
  if (! arg.is_scalar_type())  {
    return OCT_IF_ERR_TYPE_MISSMATCH_SCALAR;
  }
#if (OCTAVE_MAJOR_VERSION == 4) && (OCTAVE_MINOR_VERSION == 2)
  if (! arg.is_numeric_type()) {
#else
  if (! arg.isnumeric())  {
#endif
    return OCT_IF_ERR_TYPE_MISSMATCH_NUMERIC;
  }
#ifdef INT64_VALUE_DEFINED
  val =  arg.int64_value();
#else
  val =  arg.long_value();
#endif
  return 0;  
}
int get_arg_int( const octave_value& arg, int& val)
{
  int64_t tmp64v;
  int code = get_arg_int64( arg, tmp64v);
  if( 0 == code ) {
    if( INT_MIN > tmp64v || tmp64v > INT_MAX )  {
      return OCT_IF_ERR_ARG_INVALID_RANGE;
    }
    val = (int)tmp64v;
  }
  return code;
}
int get_arg_uint( const octave_value& arg, unsigned int& val)
{
  int64_t tmp64v;
  int code = get_arg_int64( arg, tmp64v);
  if( 0 == code ) {
    if( 0 > tmp64v || tmp64v > UINT_MAX )  {
      return OCT_IF_ERR_ARG_INVALID_RANGE;
    }
    val = (unsigned int)tmp64v;
  }
  return code;
}
int get_arg_short( const octave_value& arg, short& val)
{
  int64_t tmp64v;
  int code = get_arg_int64( arg, tmp64v);
  if( 0 == code ) {
    if( SHRT_MIN > tmp64v || tmp64v > SHRT_MAX )  {
      return OCT_IF_ERR_ARG_INVALID_RANGE;
    }
    val = (short)tmp64v;
  }
  return code;
}
int get_arg_ushort( const octave_value& arg, unsigned short& val)
{
  int64_t tmp64v;
  int code = get_arg_int64( arg, tmp64v);
  if( 0 == code ) {
    if( 0 > tmp64v || tmp64v > USHRT_MAX )  {
      return OCT_IF_ERR_ARG_INVALID_RANGE;
    }
    val = (unsigned short)tmp64v;
  }
  return code;
}

int get_arg_string( const octave_value& arg, std::string& val)
{
  if ( !arg.is_string()) {
    return OCT_IF_ERR_TYPE_MISSMATCH_STRING;
  }
  val =  arg.string_value();
  return 0; 
}
int get_arg_bool( const octave_value& arg, bool& val)
{
  if (! arg.is_scalar_type())  {
    return OCT_IF_ERR_TYPE_MISSMATCH_SCALAR;
  }
#if (OCTAVE_MAJOR_VERSION == 4) && (OCTAVE_MINOR_VERSION == 2)
  if (! arg.is_bool_type())  {
#else
  if (! arg.islogical())  {
#endif
    return OCT_IF_ERR_TYPE_MISSMATCH_BOOL;
  }
  
  val = arg.bool_value();
  return 0; 
}
void params_base_initial( PARAMS_BASE& base )
{
  base.paramCounts = 0;
  base.paramItem = NULL;
  base.paramVal = NULL;
  base.paramType = NULL;
}
void params_base_free( PARAMS_BASE& base )
{
  if( NULL != base.paramItem ) {
    for( int n=0 ; n<base.paramCounts; n++ ) {
       if( NULL != base.paramItem[n] ) delete [] base.paramItem[n];
    }
    delete [] base.paramItem;
  }
  if( NULL != base.paramVal ) {
    for( int n=0 ; n<base.paramCounts; n++ ) {
       if( NULL != base.paramVal[n] ) delete [] base.paramVal[n];
    }
    delete [] base.paramVal;
  }
  if( NULL != base.paramType ) delete [] base.paramType;

  params_base_initial(base);

}

int params_from_octave(const octave_map& params, PARAMS_BASE& base)
{
  std::string pname = "Pname";
  std::string pval = "Pvalue";
  std::string ptype = "Ptype";
  Cell namelist;
  Cell val_list;
  Cell typelist;
  char** param_item=NULL;
  char** param_val=NULL;
  int*   param_type=NULL;

  int code = 0;
  int params_nfields = params.nfields();  // 3
  int params_count = params.length();
// octave_stdout <<"params_nfields:" << params_nfields << "\n";
// octave_stdout <<"params_count:" << params_count << "\n";
    
  if( 0 == params_nfields && 0 == params_count ) {
    return 0;
  }   
    
  octave_value tmp_name;
  octave_value tmp_val;
  octave_value tmp_type;
  for( ;; ) {
    if( 3 == params_nfields ) {
      if( !params.isfield(pname) || !params.isfield(pval) || !params.isfield(ptype) ) {
        code = OCT_IF_ERR_PARAMETER_TYPE_MISSMATCH_STRUCT;   break;
      }
      // vfPʂ̂Ƃ肾
      namelist = params.contents(pname);
      val_list = params.contents(pval);
      typelist = params.contents(ptype);
      
      param_item= new char*[params_count];
      if( NULL == param_item ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break;  }
      for( int n=0 ; n<params_count; n++ ) param_item[n] = NULL;
      param_val= new char*[params_count];
      if( NULL == param_val ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break;  }
      for( int n=0 ; n<params_count; n++ ) param_val[n] = NULL;
      param_type= new int[params_count];
      if( NULL == param_type ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break;  }
      for( int n=0 ; n<params_count; n++ ) {
        tmp_name= namelist.checkelem(n);
        if( !tmp_name.is_string() ) {
          code = OCT_IF_ERR_PARAMETER_PNAME_TYPE_MISSMATCH; break;
        }
        param_item[n] = new char[tmp_name.string_value().length()+1];
        if( NULL == param_item[n] ) {
          code = OCT_RTR_ERR_MEMORY_ALLOCATE; break;
        }
        strcpy(param_item[n], tmp_name.string_value().c_str());

        tmp_val = val_list.checkelem(n);
        if( !tmp_val.is_string() ) {
          code = OCT_IF_ERR_PARAMETER_PVALUE_TYPE_MISSMATCH; break;
        }
        param_val[n] = new char[tmp_val.string_value().length()+1];
        if( NULL == param_val[n] ) {
          code = OCT_RTR_ERR_MEMORY_ALLOCATE; break;
        }
        strcpy(param_val[n], tmp_val.string_value().c_str());
        
        tmp_type = typelist.checkelem(n);
#if (OCTAVE_MAJOR_VERSION == 4) && (OCTAVE_MINOR_VERSION == 2)
        if( !tmp_type.is_scalar_type() || !tmp_type.is_numeric_type()) {
#else
        if( !tmp_type.is_scalar_type() || !tmp_type.isnumeric()) {
#endif
          code = OCT_IF_ERR_PARAMETER_PTYPE_TYPE_MISSMATCH; break;
        }
        param_type[n] = tmp_type.int_value();
      }
    }
    else if( 2 == params_nfields ) {
      if( !params.isfield(pname) || !params.isfield(pval) ) {
        code = OCT_IF_ERR_PARAMETER_TYPE_MISSMATCH_STRUCT;   break;
      }
      // vfPʂ̂Ƃ肾
      namelist = params.contents(pname);
      val_list = params.contents(pval);
      
      param_item= new char*[params_count];
      if( NULL == param_item ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break;  }
      param_val= new char*[params_count];
      if( NULL == param_val ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break;  }
      param_type= new int[params_count];
      if( NULL == param_type ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break;  }
      for( int n=0 ; n<params_count; n++ ) {
        param_item[n] = param_val[n] = NULL;
      }
      
      for( int n=0 ; n<params_count; n++ ) {
        tmp_name= namelist.checkelem(n);
        if( !tmp_name.is_string() ) {
          code = OCT_IF_ERR_PARAMETER_PNAME_TYPE_MISSMATCH; break;
        }
        param_item[n] = new char[tmp_name.string_value().length()+1];
        if( NULL == param_item[n] ) {
          code = OCT_RTR_ERR_MEMORY_ALLOCATE; break;
        }
        strcpy(param_item[n], tmp_name.string_value().c_str());
        
        tmp_val= val_list.checkelem(n);
        param_type[n] = octave2param_type( tmp_val);
        if( TYPE_UNDEF == param_type[n] ) {
          code = OCT_IF_ERR_PARAMETER_PVALUE_TYPE_NO_SUPPORT; break;
        }
        param_val[n] = octave2param_string( tmp_val);
        if( NULL == param_val[n] ) {
          code = OCT_RTR_ERR_MEMORY_ALLOCATE; break;
        }
      }
    }
    else {
      code = OCT_IF_ERR_PARAMETER_TYPE_MISSMATCH_STRUCT;   break;
    }
    break;      
  } // End of for( ;; )
  base.paramCounts = params_count;
  base.paramItem = param_item;
  base.paramVal = param_val;
  base.paramType = param_type;
  
  if( 0 != code ) params_base_free( base );
    
  return code;
}
int octave2param_type( const octave_value& val)
{
  int itype = TYPE_UNDEF;
  if( val.is_scalar_type() ) {
#if (OCTAVE_MAJOR_VERSION == 4) && (OCTAVE_MINOR_VERSION == 2)
    if( val.is_numeric_type() ) {
#else
    if( val.isnumeric() ) {
#endif
      if( val.is_double_type() ) {
        itype=TYPE_DOUBLE;
      }
#if (OCTAVE_MAJOR_VERSION == 4) && (OCTAVE_MINOR_VERSION == 2)
      else if( val.is_float_type() ) {
#else
      else if( val.isfloat() ) {
#endif
        itype=TYPE_FLOAT;
      }
      else if( val.is_int8_type() ) {
        itype=TYPE_BYTE;
      }
      else if( val.is_int16_type() ) {
        itype=TYPE_SHORT;
      }
      else if( val.is_int32_type() ) {
        itype=TYPE_INT;
      }
      else if( val.is_int64_type() ) {
        itype=TYPE_INT64;
      }
      else {
        ;
      }
    }
    else if( val.is_string() ) {
      itype=TYPE_STRING;
    }
  }
  return itype;
}
char* octave2param_string( const octave_value& val)
{
  char* buff = NULL;
  if( val.is_scalar_type() ) {
#if (OCTAVE_MAJOR_VERSION == 4) && (OCTAVE_MINOR_VERSION == 2)
    if( val.is_numeric_type() ) {
#else
    if( val.isnumeric() ) {
#endif
      if( val.is_double_type() ) {
        buff = new char[32];
        if( NULL != buff ) {
          sprintf(buff,"%.15g",val.double_value());
        }
      }
#if (OCTAVE_MAJOR_VERSION == 4) && (OCTAVE_MINOR_VERSION == 2)
      else if( val.is_float_type() ) {
#else
      else if( val.isfloat() ) {
#endif
        buff = new char[16];
        if( NULL != buff ) {
          sprintf(buff,"%.8g",val.float_value());
        }
      }
      else if( val.is_int8_type() ) {
        buff = new char[8];
        if( NULL != buff ) {
          sprintf(buff,"%d",val.int_value());
        }
      }
      else if( val.is_int16_type() ) {
        buff = new char[8];
        if( NULL != buff ) {
          sprintf(buff,"%hd",val.short_value());
        }
      }
      else if( val.is_int32_type() ) {
        buff = new char[16];
        if( NULL != buff ) {
          sprintf(buff,"%d",val.int_value());
        }
      }
      else if( val.is_int64_type() ) {
        buff = new char[32];
        if( NULL != buff ) {
          sprintf(buff,"%lld",val.int64_value());
        }
      }
      else {
        ;
      }
    }
    else if( val.is_string() ) {
      buff = new char[val.string_value().length()+1];
      if( NULL != buff ) {
        strcpy(buff, val.string_value().c_str());
      }
    }
  }
  return buff;
}
